// src/async-cache-dedupe.decorator.ts
import { Inject } from "@nestjs/common";

// src/constants/cache.constant.ts
var CACHE = Symbol("CACHE");

// src/async-cache-dedupe.decorator.ts
function AsyncCacheDedupe(options = {}) {
  const injectCache = Inject(CACHE);
  return (target, propertyKey, descriptor) => {
    const originalMethod = descriptor.value;
    injectCache(target, "cache");
    const key = `${target.constructor.name}_${propertyKey}`;
    descriptor.value = function(...args) {
      const cache = this["cache"];
      if (!cache[key]) {
        cache.define(key, options, (args2) => {
          return originalMethod.apply(this, args2);
        });
      }
      return cache[key](args);
    };
  };
}

// src/async-cache-dedupe.module.ts
import { createCache } from "async-cache-dedupe";

// src/constants/cache-module-options.constant.ts
var CACHE_MODULE_OPTIONS = Symbol("CACHE_MODULE_OPTIONS");

// src/async-cache-dedupe.module.ts
var AsyncCacheDedupeModule = class _AsyncCacheDedupeModule {
  static forRoot(options = {}) {
    const cacheProvider = {
      provide: CACHE,
      useValue: createCache(options.cacheOptions)
    };
    return {
      module: _AsyncCacheDedupeModule,
      providers: [
        {
          provide: CACHE_MODULE_OPTIONS,
          useValue: options
        },
        cacheProvider
      ],
      exports: [cacheProvider],
      global: true
    };
  }
};
export {
  AsyncCacheDedupe,
  AsyncCacheDedupeModule
};
